%   Proposed CS Preconditioning
%   Reconstruction Experiments

close all;clear all;

addpath ./Dictionaries
addpath ./Preconditioning

MAXITER = 100;
Exper = 2%500; % number of experiments
experRange = 10:10:20;
d = 64;
K = 128
N = 255;
solver = 'OMP';
proj = 'bernoulli'%
dict = 'haardct';

sp = zeros(Exper, length(experRange));
j=0;
for si=experRange
    disp('...................')
    disp(['Dictionary Size: ', num2str(d), 'X', num2str(N)]);
    j = j+1;    
    for x=1:1:Exper      
        disp(['Cardinality:', num2str(si), '(', num2str(x), ')']);
        disp('...................')  
        
        %THE SIGNAL
        D = GetDictionary(K, N, dict); %%
        P = GetDictionary(d, K, proj);
        Phi = P*D;
        [sparse_v support]= CreateTestVector(N, si, 1);            
        y = Phi*sparse_v;
        
        %PRECONDITIONING PROPOSED
        [P BPhi z] = ProposedPreconditioning(Phi, d, MAXITER, y);
%         BPhi = Phi; z = y; %ORIGINAL WITHOUT PRECONDITIONING
        
        %RECONSTRUCTION
% OMP
         [coeffs, recIndexes] = myOMP(z, BPhi);
         arec = ReconstructGivenSupport(Phi, y, recIndexes, N);

%EVALUATION
        recIndexes=find(abs(arec)>1e-3)';
        success = ReconstructionSupportSuccess(support, recIndexes);
               
%STORE RESULTS
        sp(x,j) = sp(x,j) + success;
    end    
end
sp = mean(sp);

figure; plot(experRange, sp, '.-'); 
tilt = ['CS Preconditioning: ', proj, ' projections, ', dict, ' dictionary, ', solver, ' reconstruction', ]
title([tilt,  ' (', num2str(Exper), ' experiments)'])
xlabel('support size')
ylabel('support success (percentage)')



