%   Proposed Sparse Reconstruction Experiments
%   with SensingOMP

close all;clear all;

addpath ./Dictionaries
addpath ./Preconditioning


MAXITER = 100;
Exper = 2%500; % number of experiments
experRange = 20:5:30;
d = 64;
N = 128;
dict = 'random';

sp = zeros(Exper, length(experRange));
j=0;
for si=experRange
    disp('...................')
    disp(['Dictionary Size: ', num2str(d), 'X', num2str(N)]);
    j = j+1;    
    for x=1:1:Exper      
        disp(['Support size:', num2str(si), '(', num2str(x), ')']);
        disp('...................')  
        
        %THE SIGNAL
        Phi = GetDictionary(d, N, dict); %
        [sparse_v support]= CreateTestVector(N, si, 1);            
        y = Phi*sparse_v;
        
        %PRECONDITIONING PROPOSED
        [P BPhi z] = ProposedPreconditioning(Phi, d, MAXITER, y);
        
        %RECONSTRUCTION
%1. Sensing OMP
        [coeffs, recIndexes] = SensingOMP(z, BPhi, BPhi);
        arec = ReconstructGivenSupport(Phi, y, recIndexes, N);      
        
        %EVALUATION
        recIndexes=find(abs(arec)>1e-3)';
        success = ReconstructionSupportSuccess(support, recIndexes);
                
        %STORE RESULTS
        sp(x,j) = sp(x,j) + success;
    end
end
sp = mean(sp);


figure; plot(experRange, sp, '.-'); 
tilt =['SPARSE-PREC: ', dict, ' dictionary'];
title([tilt, ' (', num2str(Exper), ' experiments)']);
xlabel('support size')
ylabel('support success (percentage)')

